@@ -201,6 +201,7 @@ def command_flash (args):
 
 	# Load in all binaries
 	binary = bytes()
+	count = 0
 	for binary_name in args.binary:
 		# check that file isn't a `.hex` file
 		if binary_name.endswith('.hex'):
@@ -211,21 +212,29 @@ def command_flash (args):
 		# add contents to binary
 		with open(binary_name, 'rb') as f:
 			binary += f.read()
+		count += 1
 
 	# Flash the binary to the chip
 	tock_loader = TockLoader(args)
 	tock_loader.open()
 
-	logging.status('Flashing binar(y|ies) to board...')
+	plural = 'y'
+	if count > 1:
+		plural = 'ies'
+	logging.status('Flashing binar{} to board...'.format(plural))
 	tock_loader.flash_binary(binary, args.address)
 
 
 def command_read (args):
-	# Read the correct flash from the chip
+	'''
+	Read the correct flash range from the chip.
+	'''
 	tock_loader = TockLoader(args)
 	tock_loader.open()
 
 	logging.status('Reading flash from the board...')
+	logging.status('  Address: {:#x}'.format(args.address))
+	logging.status('  Length:  {} bytes'.format(args.length))
 	tock_loader.read_flash(args.address, args.length)
 
 
@@ -270,16 +279,26 @@ def command_inspect_tab (args):
 
 	logging.status('Inspecting TABs...')
 	for tab in tabs:
+		# Print the basic information that is true about the TAB and all
+		# contained TBF binaries.
 		print(tab)
 
-		# If the user asked for the crt0 header, display that for each
-		# architecture.
-		if args.crt0_header:
-			print('  crt0 header')
-			archs = tab.get_supported_architectures()
-			for arch in archs:
-				print('    {}'.format(arch))
-				print(textwrap.indent(tab.get_crt0_header_str(arch), '      '))
+		# Ask the user if they want to see more detail about a certain TBF.
+		tbf_names = tab.get_tbf_names()
+		index = helpers.menu(tbf_names+['None'],
+		                     return_type='index',
+		                     title='Which TBF to inspect further?')
+		if index < len(tbf_names):
+			print('')
+			print('{}:'.format(tbf_names[index]))
+			app = tab.extract_tbf(tbf_names[index])
+			print(textwrap.indent(str(app.get_header()), '  '))
+
+			# If the user asked for the crt0 header, display that for the
+			# architecture
+			if args.crt0_header:
+				print('  crt0 header')
+				print(textwrap.indent(app.get_crt0_header_str() , '    '))
 
 		print('')
 
@@ -548,11 +567,11 @@ def main ():
 		parents=[parent, parent_channel],
 		help='Read arbitrary flash memory')
 	flash.set_defaults(func=command_read)
-	flash.add_argument('--address', '-a',
+	flash.add_argument('address',
 		help='Address to read from',
 		type=lambda x: int(x, 0),
 		default=0x30000)
-	flash.add_argument('--length', '-l',
+	flash.add_argument('length',
 		help='Number of bytes to read',
 		type=lambda x: int(x, 0),
 		default=512)
