@@ -9,6 +9,7 @@ import binascii
 import contextlib
 import copy
 import functools
+import itertools
 import logging
 import os
 import platform
@@ -17,7 +18,9 @@ import textwrap
 import time
 
 from . import helpers
-from .app import App
+from .app_installed import InstalledApp
+from .app_padding import PaddingApp
+from .app_tab import TabApp
 from .board_interface import BoardInterface
 from .bootloader_serial import BootloaderSerial
 from .exceptions import TockLoaderException
@@ -68,6 +71,7 @@ class TockLoader:
 	    'nrf52dk': {'size_minimum': 4096},
 	    'edu-ciaa': {'cmd_flags': {'bundle_apps': True,
 	                               'openocd': True}},
+	    'arty': {'size_minimum': 0x10000},
 	}
 
 
@@ -136,7 +140,7 @@ class TockLoader:
 		'''
 		Add or update TABs on the board.
 
-		- `replace` can be either "yes", "no", or "only"
+		- `replace` can be "yes", "no", or "only"
 		- `erase` if true means erase all other apps before installing
 		'''
 		# Check if we have any apps to install. If not, then we can quit early.
@@ -179,7 +183,7 @@ class TockLoader:
 			if replace == 'yes' or replace == 'only':
 				for existing_app in existing_apps:
 					for replacement_app in replacement_apps:
-						if existing_app.name == replacement_app.name:
+						if existing_app.get_name() == replacement_app.get_name():
 							resulting_apps.append(copy.deepcopy(replacement_app))
 							changed = True
 							break
@@ -193,7 +197,7 @@ class TockLoader:
 				if replace == 'yes':
 					for replacement_app in replacement_apps:
 						for resulting_app in resulting_apps:
-							if replacement_app.name == resulting_app.name:
+							if replacement_app.get_name() == resulting_app.get_name():
 								break
 						else:
 							# We did not find the name in the resulting apps.
@@ -231,23 +235,23 @@ class TockLoader:
 					raise TockLoaderException('No apps are installed on the board')
 				elif len(apps) == 1:
 					# If there's only one app, delete it
-					app_names = [apps[0].name]
+					app_names = [apps[0].get_name()]
 					logging.info('Only one app on board.')
 				else:
 					options = ['** Delete all']
-					options.extend([app.name for app in apps])
+					options.extend([app.get_name() for app in apps])
 					name = helpers.menu(options,
 							return_type='value',
 							prompt='Select app to uninstall ',
 							title='There are multiple apps currently on the board:')
 					if name == '** Delete all':
-						app_names = [app.name for app in apps]
+						app_names = [app.get_name() for app in apps]
 					else:
 						app_names = [name]
 
-			print('Attempting to uninstall:')
+			logging.status('Attempting to uninstall:')
 			for app_name in app_names:
-				print('  - {}'.format(app_name))
+				logging.status('  - {}'.format(app_name))
 
 			# Remove the apps if they are there
 			removed = False
@@ -255,7 +259,7 @@ class TockLoader:
 			for app in apps:
 				# Only keep apps that are not marked for uninstall or that
 				# are sticky (unless force was set)
-				if app.name not in app_names or (app.is_sticky() and not self.args.force):
+				if app.get_name() not in app_names or (app.is_sticky() and not self.args.force):
 					keep_apps.append(app)
 				else:
 					removed = True
@@ -264,12 +268,12 @@ class TockLoader:
 			# sticky, or if we are removing apps because --force was provided.
 			if not self.args.force:
 				for app in apps:
-					if app.name in app_names and app.is_sticky():
+					if app.get_name() in app_names and app.is_sticky():
 						logging.info('Not removing app "{}" because it is sticky.'.format(app))
 						logging.info('To remove this you need to include the --force option.')
 			else:
 				for app in apps:
-					if app.name in app_names and app.is_sticky():
+					if app.get_name() in app_names and app.is_sticky():
 						logging.info('Removing sticky app "{}" because --force was used.'.format(app))
 
 			# Check if we actually have any work to do.
@@ -280,13 +284,14 @@ class TockLoader:
 
 				logging.status('Uninstall complete.')
 
-				# And let the user know the state of the world now that we're done
-				apps = self._extract_all_app_headers()
-				if len(apps):
-					print('After uninstall, remaining apps on board: ', end='')
-					self._print_apps(apps, verbose=False, quiet=True)
-				else:
-					print('After uninstall, no apps on board.')
+				if self.args.debug:
+					# And let the user know the state of the world now that we're done
+					apps = self._extract_all_app_headers()
+					if len(apps):
+						logging.info('After uninstall, remaining apps on board: ', end='')
+						self._print_apps(apps, verbose=False, quiet=True)
+					else:
+						loggging.info('After uninstall, no apps on board.')
 
 			else:
 				raise TockLoaderException('Could not find any apps on the board to remove.')
@@ -321,11 +326,11 @@ class TockLoader:
 					address = self.channel.get_apps_start_address()
 					self.channel.erase_page(address)
 
-					print('All apps have been erased.')
+					logging.info('All apps have been erased.')
 				else:
 					self._reshuffle_apps(keep_apps)
 
-					print('After erasing apps, remaining apps on board: ', end='')
+					logging.info('After erasing apps, remaining apps on board: ', end='')
 					self._print_apps(apps, verbose=False, quiet=True)
 
 
@@ -345,28 +350,28 @@ class TockLoader:
 			# User did not specify apps. Pick from list.
 			if len(app_names) == 0:
 				options = ['** All']
-				options.extend([app.name for app in apps])
+				options.extend([app.get_name() for app in apps])
 				name = helpers.menu(options,
 						return_type='value',
 						prompt='Select app to configure ',
 						title='Which apps to configure?')
 				if name == '** All':
-					app_names = [app.name for app in apps]
+					app_names = [app.get_name() for app in apps]
 				else:
 					app_names = [name]
 
 			# Configure all selected apps
 			changed = False
 			for app in apps:
-				if app.name in app_names:
-					app.tbfh.set_flag(flag_name, flag_value)
+				if app.get_name() in app_names:
+					app.get_header().set_flag(flag_name, flag_value)
 					changed = True
 
 			if changed:
 				self._reshuffle_apps(apps)
-				print('Set flag "{}" to "{}" for apps: {}'.format(flag_name, flag_value, ', '.join(app_names)))
+				logging.info('Set flag "{}" to "{}" for apps: {}'.format(flag_name, flag_value, ', '.join(app_names)))
 			else:
-				print('No matching apps found. Nothing changed.')
+				logging.info('No matching apps found. Nothing changed.')
 
 
 	def list_attributes (self):
@@ -384,7 +389,7 @@ class TockLoader:
 
 	def set_attribute (self, key, value):
 		'''
-		Download all attributes stored on the board.
+		Change an attribute stored on the board.
 		'''
 		# Do some checking
 		if len(key.encode('utf-8')) > 8:
@@ -639,9 +644,171 @@ class TockLoader:
 	def _reshuffle_apps (self, apps):
 		'''
 		Given an array of apps, some of which are new and some of which exist,
-		sort them in flash so they are in descending size order. Then write
-		these apps to flash.
+		sort them so we can write them to flash.
+
+		This function is really the driver of tockloader, and is responsible for
+		setting up applications in a way that can be successfully used by the
+		board.
 		'''
+
+		#
+		# JUNE 2020: This function can be really complicated (balancing apps
+		# compiled for a fixed address, MPU alignment concerns, ordering
+		# concerns, handling apps from TABs and already installed etc.) and by
+		# no means is the current implementation arriving at an optimal
+		# solution. An interested contributor could probably find many
+		# improvements and optimizations.
+		#
+
+
+		# Get where the apps live in flash.
+		address = self.channel.get_apps_start_address()
+
+
+		# First, we are going to split the work into two cases: do we have any
+		# app that is compiled for a fixed address, or not? Likely, there won't
+		# be platforms that have mixed fixed address apps and PIC apps. This
+		# split simplifies things, but a better algorithm would not have this
+		# split.
+		is_fixed_address_app = False
+		for app in apps:
+			if app.has_fixed_addresses():
+				is_fixed_address_app = True
+
+		if is_fixed_address_app:
+			#
+			# This is the fixed addresses case
+			#
+
+			def brad_sort (slices):
+				'''
+				Get an ordering of apps where the fixed start addresses are
+				respected and the apps do not overlap.
+
+				Brute force method!
+				'''
+				def is_valid (slices):
+					'''
+					Check if the list of app regions (slices) can fit correctly.
+					'''
+					slices = list(slices)
+					slices.sort(key=lambda x: x[0])
+					end = 0
+					for s in slices:
+						if s[0] < end:
+							return False
+						end = s[0] + s[1]
+					return True
+
+				# Get a list of all possible orderings.
+				options = itertools.product(*slices)
+				# See if any work.
+				for o in options:
+					if is_valid(o):
+						return o
+
+				# Couldn't find a valid ordering.
+				return None
+
+			# Get a list of all possible start and length pairs for each app to
+			# flash. Also keep around the index of the app in original array.
+			slices = []
+			for i,app in enumerate(apps):
+				starting_addresses = app.get_fixed_addresses_flash()
+				app_slices = []
+				for sa in starting_addresses:
+					if sa < address:
+						# Can't use an app below the start of apps address.
+						continue
+					app_slices.append([sa, app.get_size(), i])
+				slices.append(app_slices)
+
+			# See if we can find an ordering that works.
+			valid_order = brad_sort(slices)
+			if valid_order == None:
+				logging.error('Unable to find a valid sort order to flash apps.')
+				return
+
+			# Get sorted apps array.
+			logging.debug('Found sort order:')
+			sorted_apps = []
+			for order in valid_order:
+				app = apps[order[2]]
+				logging.debug('  App "{}" at address {:#x}'.format(app.get_name(), order[0]))
+				sorted_apps.append(app)
+			apps = sorted_apps
+
+			# Iterate all of the apps, and see if we can make this work based on
+			# having apps compiled for the correct addresses. If so, great! If
+			# not, error for now.
+			to_flash_apps = []
+			app_address = address
+			for app in apps:
+				# Ask the app where we can put it that is greater than or equal
+				# to the current address we are looking at.
+				next_loadable_address = app.get_next_loadable_address(app_address)
+				if next_loadable_address == app_address:
+					to_flash_apps.append(app)
+					app_address += app.get_size()
+
+				elif next_loadable_address != None:
+					# Need to add padding.
+					padding_app = PaddingApp(next_loadable_address - app_address)
+					to_flash_apps.append(padding_app)
+					app_address += padding_app.get_size()
+
+					to_flash_apps.append(app)
+					app_address += app.get_size()
+
+				else:
+					logging.error('Trying to find a location for app "{}"'.format(app))
+					logging.error('  Address to use is {:#x}'.format(app_address))
+					raise TockLoaderException('Could not load apps due to address mismatches')
+
+
+			# Actually write apps to the board.
+			app_address = address
+			if self.args.bundle_apps:
+				# Tockloader has been configured to bundle all apps as a single
+				# binary. Here we concatenate all apps and then call flash once.
+				#
+				# This should be compatible with all boards, but if there
+				# several existing apps they have to be re-flashed, and that
+				# could add significant overhead. So, we prefer to flash only
+				# what has changed and special case this bundle operation.
+				app_bundle = bytearray()
+				for app in to_flash_apps:
+					app_bundle += app.get_binary(app_address)
+					app_address += app.get_size()
+				logging.debug('Installing app bundle. Size: {} bytes.'.format(len(app_bundle)))
+				self.channel.flash_binary(address, app_bundle)
+			else:
+				# Flash only apps that have been modified. The only way an app
+				# would not be modified is if it was read off the board and
+				# nothing changed.
+				for app in to_flash_apps:
+					# If we get a binary, then we need to flash it. Otherwise,
+					# the app is already installed.
+					optional_binary = app.get_binary(app_address)
+					if optional_binary:
+						self.channel.flash_binary(app_address, optional_binary)
+					app_address = app_address + app.get_size()
+
+			# Then erase the next page if we have not already rewritten all
+			# existing apps. This ensures that flash is clean at the end of the
+			# installed apps and makes sure the kernel will find the correct end
+			# of applications.
+			self.channel.erase_page(app_address)
+
+			# Handled fixed address case, do not continue on to run the
+			# non-fixed address case.
+			return
+
+
+		#
+		# This is the normal PIC case
+		#
+
 		# We are given an array of apps. First we need to order them based on
 		# the ordering requested by this board (or potentially the user).
 		if self.app_options['order'] == 'size_descending':
@@ -649,9 +816,6 @@ class TockLoader:
 		else:
 			raise TockLoaderException('Unknown sort order. This is a tockloader bug.')
 
-		# Get where the apps live in flash.
-		address = self.channel.get_apps_start_address()
-
 		# Decide if we need to read the existing binary from flash into memory.
 		# We can need to do this for various reasons:
 		#
@@ -669,28 +833,23 @@ class TockLoader:
 		#
 		# So, we iterate through all apps and read them into memory if we are
 		# doing an erase and re-flash cycle or if the app has moved.
-		start_address = address
+		app_address = address
 		for app in apps:
 			# If we do not already have a binary, and any of the conditions are
 			# met, we need to read the app binary from the board.
 			if (not app.has_app_binary()) and \
 			   (self.args.bundle_apps or
-			   	app.address != start_address or
+			   	app.get_address() != app_address or
 			   	app.is_modified()):
 					logging.debug('Reading app {} binary from board.'.format(app))
 					entire_app = self.channel.read_range(app.address, app.get_size())
 					in_flash_tbfh = TBFHeader(entire_app)
 					app.set_app_binary(entire_app[in_flash_tbfh.get_header_size():])
 
-			# Only if the app moved do we save the new address.
-			if app.address != start_address:
-				app.set_address(start_address)
+			app_address += app.get_size()
 
-			start_address += app.get_size()
-
-		# Need to know the address after the last app (`end`) so that we can
-		# make sure to clear the flash where the TBF header would go.
-		end = address
+		# Need to know the address we are putting each app at.
+		app_address = address
 
 		# Actually write apps to the board.
 		if self.args.bundle_apps:
@@ -703,25 +862,28 @@ class TockLoader:
 			# and special case this bundle operation.
 			app_bundle = bytearray()
 			for app in apps:
-				app_bundle += app.get_binary()
-				end += app.get_size()
+				app_bundle += app.get_binary(app_address)
+				app_address += app.get_size()
 			logging.debug('Installing app bundle. Size: {} bytes.'.format(len(app_bundle)))
 			self.channel.flash_binary(address, app_bundle)
 		else:
 			# Flash only apps that have been modified. The only way an app would
 			# not be modified is if it was read off the board and nothing
-			# changed. An app from a TAB will be modified since it will have a
-			# new address.
+			# changed.
 			for app in apps:
-				if app.is_modified():
-					self.channel.flash_binary(app.address, app.get_binary())
-				end = app.address + app.get_size()
+				# If we get a binary, then we need to flash it. Otherwise,
+				# the app is already installed.
+				optional_binary = app.get_binary(app_address)
+				if optional_binary:
+					logging.debug('Flashing app {} binary to board.'.format(app))
+					self.channel.flash_binary(app_address, optional_binary)
+				app_address = app_address + app.get_size()
 
 		# Then erase the next page if we have not already rewritten all existing
 		# apps. This ensures that flash is clean at the end of the installed
 		# apps and makes sure the kernel will find the correct end of
 		# applications.
-		self.channel.erase_page(end)
+		self.channel.erase_page(app_address)
 
 	def _extract_all_app_headers (self):
 		'''
@@ -747,14 +909,7 @@ class TockLoader:
 			tbfh = TBFHeader(flash)
 
 			if tbfh.is_valid():
-				# Get the name out of the app.
-				name_or_params = tbfh.get_app_name()
-				if isinstance(name_or_params, str):
-					name = name_or_params
-				else:
-					name = self._get_app_name(address+name_or_params[0], name_or_params[1])
-
-				app = App(tbfh, address, name)
+				app = InstalledApp(tbfh, address)
 				apps.append(app)
 
 				address += app.get_size()
@@ -771,7 +926,7 @@ class TockLoader:
 
 	def _extract_apps_from_tabs (self, tabs):
 		'''
-		Iterate through the list of TABs and create the app dict for each.
+		Iterate through the list of TABs and create the app object for each.
 		'''
 		apps = []
 
@@ -811,16 +966,6 @@ class TockLoader:
 
 		return apps
 
-	def _get_app_name (self, address, length):
-		'''
-		Retrieve bytes from the board and interpret them as a string
-		'''
-		if length == 0:
-			return ''
-
-		name_memory = self.channel.read_range(address, length)
-		return name_memory.decode('utf-8')
-
 	def _app_is_aligned_correctly (self, address, size):
 		'''
 		Check if putting an app at this address will be OK with the MPU.
@@ -875,24 +1020,21 @@ class TockLoader:
 		if not quiet:
 			# Print info about each app
 			for i,app in enumerate(apps):
-				print('[App {}]'.format(i))
+				print(helpers.text_in_box('App {}'.format(i), 52))
 
 				# Check if this app is OK with the MPU region requirements.
-				if not self._app_is_aligned_correctly(app.address, app.get_size()):
+				if not self._app_is_aligned_correctly(app.get_address(), app.get_size()):
 					print('  [WARNING] App is misaligned for the MPU')
 
 				print(textwrap.indent(app.info(verbose), '  '))
 				print('')
 
 			if len(apps) == 0:
-				print('No found apps.')
+				logging.info('No found apps.')
 
 		else:
 			# In quiet mode just show the names.
-			app_names = []
-			for app in apps:
-				app_names.append(app.name)
-			print(' '.join(app_names))
+			print(' '.join([app.get_name() for app in apps]))
 
 	def _print_attributes (self, attributes):
 		'''
